#ifndef _LV_ACTOR_H
#define _LV_ACTOR_H

#include <libvisual/lv_audio.h>
#include <libvisual/lv_video.h>
#include <libvisual/lv_palette.h>
#include <libvisual/lv_plugin.h>
#include <libvisual/lv_songinfo.h>
#include <libvisual/lv_event.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

typedef struct _VisActor VisActor;

struct _VisActor {
	LVPlugin	*plugin;

	VisVideo	*video;
	VisVideo	*transform;
	VisVideo	*fitting;
	VisPalette	*ditherpal;

	VisSongInfo	*songinfo;
	VisSongInfo	*songcompare;

	VisEventQueue	events;

	void		*private;
};

/* prototypes */
VisList *visual_actor_get_list ();
char *visual_actor_get_next_by_name (char *name);
char *visual_actor_get_prev_by_name (char *name);
int visual_actor_valid_by_name (char *name);

VisActor *visual_actor_new (char *actorname);

int visual_actor_realize (VisActor *actor);
int visual_actor_destroy (VisActor *actor);
int visual_actor_free (VisActor *actor);

int visual_actor_events_pump (VisActor *actor);

VisEventQueue *visual_actor_get_eventqueue (VisActor *actor);
VisSongInfo *visual_actor_get_songinfo (VisActor *actor);
VisPalette *visual_actor_get_palette (VisActor *actor);

int visual_actor_video_negotiate (VisActor *actor, int rundepth, int noevent, int forced);
int visual_actor_get_supported_depth (VisActor *actor);

int visual_actor_set_video (VisActor *actor, VisVideo *video);

int visual_actor_run (VisActor *actor, VisAudio *audio);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _LV_ACTOR_H */
