#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <lv_event.h>

VisEvent *visual_event_new ()
{
	VisEvent *event;

	event = malloc (sizeof (VisEvent));
	memset (event, 0, sizeof (VisEvent));

	return event;
}

int visual_event_free (VisEvent *event)
{
	if (event == NULL)
		return -1;

	free (event);

	return 0;
}

VisEventQueue *visual_event_queue_new ()
{
	VisEventQueue *eventqueue;

	eventqueue = malloc (sizeof (VisEventQueue));
	memset (eventqueue, 0, sizeof (VisEventQueue));

	eventqueue->mousestate = VISUAL_MOUSE_UP;

	return eventqueue;
}

int visual_event_queue_free (VisEventQueue *eventqueue)
{
	if (eventqueue == NULL)
		return -1;

	visual_list_destroy (&eventqueue->events, free);
	
	free (eventqueue);
	
	return 0;
}

int visual_event_queue_poll (VisEventQueue *eventqueue, VisEvent *event)
{
	VisEvent *lev;
	VisListEntry *listentry = NULL;;

	if (eventqueue->resizenew == TRUE) {
		eventqueue->resizenew = FALSE;

		memcpy (event, &eventqueue->lastresize, sizeof (VisEvent));

		return 1;
	}

	if (eventqueue->eventcount <= 0)
		return 0;
	
	lev = visual_list_next (&eventqueue->events, &listentry);
	memcpy (event, lev, sizeof (VisEvent));

	visual_event_free (lev);

	visual_list_delete (&eventqueue->events, &listentry);
	
	eventqueue->eventcount--;

	return 1;
}

int visual_event_queue_add (VisEventQueue *eventqueue, VisEvent *event)
{
	/* We've got way too much on the queue, not adding events, the important
	 * resize event got data in the eventqueue structure that makes sure it gets
	 * looked at */
	if (eventqueue->eventcount > VISUAL_EVENT_MAXEVENTS) {
		visual_event_free (event);
		return -1;
	}

	visual_list_add (&eventqueue->events, event);

	eventqueue->eventcount++;

	return 0;
}

int visual_event_queue_add_keyboard (VisEventQueue *eventqueue, int keysym, int keymod, int state)
{
	VisEvent *event;

	event = visual_event_new ();
	
	/* FIXME name to VISUAL_KEYB_DOWN and KEYB_UP */
	if (state == 0)
		event->type = VISUAL_EVENT_KEYDOWN;
	else
		event->type = VISUAL_EVENT_KEYUP;

	event->keyboard.type = event->type;
	event->keyboard.keysym.sym = keysym;
	event->keyboard.keysym.mod = keymod;

	return visual_event_queue_add (eventqueue, event);
}

int visual_event_queue_add_mousemotion (VisEventQueue *eventqueue, int x, int y)
{
	VisEvent *event;

	event = visual_event_new ();

	event->type = VISUAL_EVENT_MOUSEMOTION;

	event->mousemotion.type = event->type;

	event->mousemotion.state = eventqueue->mousestate;
	event->mousemotion.x = x;
	event->mousemotion.y = y;

	event->mousemotion.xrel = x - eventqueue->mousex;
	event->mousemotion.yrel = y - eventqueue->mousey;

	eventqueue->mousex = x;
	eventqueue->mousey = y;

	return visual_event_queue_add (eventqueue, event);;
}

int visual_event_queue_add_mousebutton (VisEventQueue *eventqueue, int button, int state)
{
	VisEvent *event;

	event = visual_event_new ();

	if (state == VISUAL_MOUSE_DOWN)
		event->type = VISUAL_EVENT_MOUSEBUTTONDOWN;
	else
		event->type = VISUAL_EVENT_MOUSEBUTTONUP;

	event->mousebutton.type = event->type;
	
	event->mousebutton.button = button;
	event->mousebutton.state = state;
	
	event->mousebutton.x = eventqueue->mousex;
	event->mousebutton.y = eventqueue->mousey;

	eventqueue->mousestate = state;

	return visual_event_queue_add (eventqueue, event);
}

int visual_event_queue_add_resize (VisEventQueue *eventqueue, VisVideo *video, int width, int height)
{
	VisEvent *event;

	event = &eventqueue->lastresize;
	
	event->type = VISUAL_EVENT_RESIZE;

	event->resize.type = event->type;

	event->resize.video = video;
	event->resize.width = width;
	event->resize.height = height;

	eventqueue->resizenew = TRUE;

	return 0;
}

int visual_event_queue_add_newsong (VisEventQueue *eventqueue, VisSongInfo *songinfo)
{
	VisEvent *event;

	event = visual_event_new ();

	event->type = VISUAL_EVENT_NEWSONG;

	event->newsong.type = event->type;
	event->newsong.songinfo = songinfo;

	return visual_event_queue_add (eventqueue, event);
}

