#ifndef _LV_EVENT_H
#define _LV_EVENT_H

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

#define VISUAL_EVENT_MAXEVENTS	256

#include <libvisual/lv_songinfo.h>
#include <libvisual/lv_video.h>
#include <libvisual/lv_list.h>
#include <libvisual/lv_keysym.h>
	
enum {
	VISUAL_EVENT_NOEVENT,
	VISUAL_EVENT_KEYDOWN,
	VISUAL_EVENT_KEYUP,
	VISUAL_EVENT_MOUSEMOTION,
	VISUAL_EVENT_MOUSEBUTTONDOWN,
	VISUAL_EVENT_MOUSEBUTTONUP,
	VISUAL_EVENT_NEWSONG,
	VISUAL_EVENT_RESIZE
};

enum {
	VISUAL_KEY_DOWN,
	VISUAL_KEY_UP
};

enum {
	VISUAL_MOUSE_DOWN,
	VISUAL_MOUSE_UP
};

typedef struct _VisEventKeyboard VisEventKeyboard;
typedef struct _VisEventMouseMotion VisEventMouseMotion;
typedef struct _VisEventMouseButton VisEventMouseButton;
typedef struct _VisEventResize VisEventResize;
typedef struct _VisEventNewSong VisEventNewSong;
typedef struct _VisEvent VisEvent;
typedef struct _VisEventQueue VisEventQueue;

struct _VisEventKeyboard {
	int		type; /* KEYDOWN, KEYUP */
	VisKeySym	keysym;
};

struct _VisEventMouseMotion {
	int	type; /* MOUSEMOTION */
	int	state; /* The current button state */
	int	x; /* Absolute motions */
	int	y;
	int	xrel; /* Relative motions */
	int	yrel;
};

struct _VisEventMouseButton {
	int	type; /* MOUSEBUTTONDOWN, MOUSEBUTTONUP */
	int	state; /* The current button state */
	int	button; /* Button index */
	int	x;
	int	y;
};

struct _VisEventResize {
	int		 type; /* RESIZE */
	VisVideo	*video;
	int		 width;
	int		 height;
};

struct _VisEventNewSong {
	int		 type;
	VisSongInfo	*songinfo;
};

struct _VisEvent {
	int			type;
	VisEventKeyboard	keyboard;
	VisEventMouseMotion	mousemotion;
	VisEventMouseButton	mousebutton;
	VisEventResize		resize;
	VisEventNewSong		newsong;
};

struct _VisEventQueue {
	VisList		 events;
	VisEvent 	 lastresize;
	int		 resizenew;;
	int		 eventcount;

	int		 mousex;
	int		 mousey;
	int		 mousestate;
};

VisEvent *visual_event_new ();
int visual_event_free (VisEvent *event);
VisEventQueue *visual_event_queue_new ();
int visual_event_queue_free (VisEventQueue *eventqueue);
int visual_event_queue_poll (VisEventQueue *eventqueue, VisEvent *event);
int visual_event_queue_add (VisEventQueue *eventqueue, VisEvent *event);
int visual_event_queue_add_keyboard (VisEventQueue *eventqueue, int keysym, int keymod, int state);
int visual_event_queue_add_mousemotion (VisEventQueue *eventqueue, int x, int y);
int visual_event_queue_add_mousebutton (VisEventQueue *eventqueue, int button, int state);
int visual_event_queue_add_resize (VisEventQueue *eventqueue, VisVideo *video, int width, int height);
int visual_event_queue_add_newsong (VisEventQueue *eventqueue, VisSongInfo *songinfo);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _LV_EVENT_H */
