#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <lv_list.h>
#include <lv_audio.h>
#include <lv_plugin.h>
#include <lv_input.h>

extern VisList *__lv_plugins_input;

static VisInputPlugin *get_input_plugin (VisInput *input);
                                                                                                                                               
static VisInputPlugin *get_input_plugin (VisInput *input)
{
	VisInputPlugin *inplugin;

	inplugin = input->plugin->plugin.inputplugin;

	return inplugin;
}

VisList *visual_input_get_list ()
{
	return __lv_plugins_input;
}

char *visual_input_get_next_by_name (char *name)
{
	return visual_plugin_get_next_by_name (visual_input_get_list (), name);
}
                                                                                                                                               
char *visual_input_get_prev_by_name (char *name)
{
	return visual_plugin_get_prev_by_name (visual_input_get_list (), name);
}

int visual_input_valid_by_name (char *name)
{
	if (_lv_plugin_find (visual_input_get_list (), name) == NULL)
		return FALSE;
	else
		return TRUE;
}

VisInput *visual_input_new (char *inputname)
{
	VisInput *input;
	VisPluginRef *ref;

	if (__lv_plugins_input == NULL && inputname != NULL)
		return NULL;

	input = malloc (sizeof (VisInput));
	memset (input, 0, sizeof (VisInput));

	input->audio = visual_audio_new ();
	
	if (inputname == NULL)
		return input;
	
	ref = _lv_plugin_find (__lv_plugins_input, inputname);
	
	input->plugin = _lv_plugin_load (ref);

	return input;
}

int visual_input_realize (VisInput *input)
{
	if (input == NULL)
		return -1;

	if (input->plugin != NULL && input->callback == NULL)
		_lv_plugin_realize (input->plugin);

	return 0;
}

int visual_input_destroy (VisInput *input)
{
	if (input == NULL)
		return -1;

	if (input->plugin != NULL)
		_lv_plugin_unload (input->plugin);

	return visual_input_free (input);
		
}

int visual_input_free (VisInput *input)
{
	if (input == NULL)
		return -1;

	if (input->audio != NULL)
		visual_audio_free (input->audio);
	
	free (input);
	
	return 0;
}

int visual_input_set_callback (VisInput *input, input_upload_callback_func_t callback, void *private)
{
	if (input == NULL)
		return -1;

	input->callback = callback;
	input->private = private;

	return 0;
}

int visual_input_run (VisInput *input)
{
	VisInputPlugin *inplugin;
	
	if (input == NULL)
		return -1;

	if (input->callback == NULL) {
		inplugin = get_input_plugin (input);	

		inplugin->upload (inplugin, input->audio);
	} else
		input->callback (input, input->audio, input->private);

	visual_audio_analyze (input->audio);

	return 0;
}

