#ifndef _LV_LIST_H
#define _LV_LIST_H

#include <libvisual/lv_common.h>

#if defined(__FreeBSD__) || defined(__OpenBSD__)
#include <sys/queue.h>
#endif

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

struct _VisListEntry {
	struct _VisListEntry	*prev, *next;
	void			*data;
};
typedef struct _VisListEntry VisListEntry;

struct _VisList {
	struct _VisListEntry	*head, *tail;
	int			count;
};
typedef struct _VisList VisList;

typedef int (*visual_list_sort_func_t)();
typedef int visual_list_sort_t;
typedef void (*visual_list_destroy_func_t)();
typedef void visual_list_destroy_t;

static inline void *visual_list_next(VisList *list, VisListEntry **le)
{
	if (!*le)
		*le = list->head;
	else
		*le = (*le)->next;

	if (*le)
		return (*le)->data;

	return NULL;
}

static inline void *visual_list_prev(VisList *list, VisListEntry **le)
{
	if (!*le)
		*le = list->tail;
	else
		*le = (*le)->prev;

	if (*le)
		return (*le)->data;

	return NULL;
}

/* prototypes */
int visual_list_init (VisList *list);
int visual_list_free (VisList *list);
int visual_list_add_at_begin (VisList *list, void *data);
int visual_list_add (VisList *list, void *data);
int visual_list_count (VisList *list);
void *visual_list_get (VisList *list, int index);
void visual_list_delete (VisList *list, VisListEntry **le);
void visual_list_exchange_item (VisList *list, VisListEntry **a, VisListEntry **b);
VisListEntry *visual_list_find (VisList *list, void * ptr);
void visual_list_find_item_id (VisList *list, VisListEntry **le, int id);
int visual_list_insert (VisList *list, VisListEntry **le, void * data);
VisList *visual_list_new ();
int visual_list_sort (VisList *list, visual_list_sort_func_t compare);
int visual_list_destroy (VisList *list, visual_list_destroy_func_t destroyer);
int visual_list_duplicate_header (VisList *parent, VisList *child);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _LV_LIST_H */
