#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <lv_log.h>
#include <lv_palette.h>
#include <lv_morph.h>

extern VisList *__lv_plugins_morph;

static VisMorphPlugin *get_morph_plugin (VisMorph *morph);

static VisMorphPlugin *get_morph_plugin (VisMorph *morph)
{
	VisMorphPlugin *morphplugin;

	morphplugin = morph->plugin->plugin.morphplugin;

	return morphplugin;
}

VisList *visual_morph_get_list ()
{
	return __lv_plugins_morph;
}

char *visual_morph_get_next_by_name (char *name)
{
	return visual_plugin_get_next_by_name (visual_morph_get_list (), name);
}
                                                                                                                                               
char *visual_morph_get_prev_by_name (char *name)
{
	return visual_plugin_get_prev_by_name (visual_morph_get_list (), name);
}

int visual_morph_valid_by_name (char *name)
{
	if (_lv_plugin_find (visual_morph_get_list (), name) == NULL)
		return FALSE;
	else
		return TRUE;
}

VisMorph *visual_morph_new (char *morphname)
{
	VisMorph *morph;
	VisPluginRef *ref;

	if (__lv_plugins_morph == NULL && morphname != NULL) {
		visual_log (VISUAL_LOG_LEVEL_CRITICAL, "the plugin list is NULL");
		return NULL;
	}

	morph = malloc (sizeof (VisMorph));
	memset (morph, 0, sizeof (VisMorph));

	if (morphname == NULL)
		return morph;

	ref = _lv_plugin_find (__lv_plugins_morph, morphname);

	morph->plugin = _lv_plugin_load (ref);

	return morph;
}

int visual_morph_realize (VisMorph *morph)
{
	if (morph == NULL) {
		visual_log (VISUAL_LOG_LEVEL_CRITICAL, "the morph is NULL");
		return -1;
	}

	if (morph->plugin == NULL) {
		visual_log (VISUAL_LOG_LEVEL_CRITICAL, "the morph->plugin element is NULL");
		return -1;
	}

	_lv_plugin_realize (morph->plugin);

	return 0;
}

int visual_morph_destroy (VisMorph *morph)
{
	if (morph == NULL)
		return -1;

	if (morph->plugin != NULL)
		_lv_plugin_unload (morph->plugin);

	return visual_morph_free (morph);
}

int visual_morph_free (VisMorph *morph)
{
	if (morph == NULL)
		return -1;
	
	free (morph);

	return 0;
}

int visual_morph_get_supported_depth (VisMorph *morph)
{
	VisMorphPlugin *morphplugin;

	if (morph == NULL)
		return -1;

	morphplugin = get_morph_plugin (morph);

	return morphplugin->depth;
}

int visual_morph_set_video (VisMorph *morph, VisVideo *video)
{
	if (morph == NULL)
		return -1;

	morph->dest = video;

	return 0;
}

int visual_morph_set_rate (VisMorph *morph, float rate)
{
	if (morph == NULL)
		return -1;

	morph->rate = rate;

	return 0;
}

VisPalette *visual_morph_get_palette (VisMorph *morph)
{
	return &morph->morphpal;
}

int visual_morph_run (VisMorph *morph, VisAudio *audio, VisVideo *src1, VisVideo *src2)
{
	VisMorphPlugin *morphplugin;

	if (morph == NULL)
		return -1;

	morphplugin = get_morph_plugin (morph);

	if (morphplugin->palette != NULL)
		morphplugin->palette (morphplugin, morph->rate, audio, &morph->morphpal, src1, src2);
	else
		visual_palette_blend (&morph->morphpal, src1->pal, src2->pal, morph->rate);

	morphplugin->apply (morphplugin, morph->rate, audio, morph->dest, src1, src2);

	return 0;
}

