#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <time.h>

#include <lv_video.h>
#include <lv_songinfo.h>

VisSongInfo *visual_songinfo_new (int interface)
{
	VisSongInfo *songinfo;

	songinfo = malloc (sizeof (VisSongInfo));
	memset (songinfo, 0, sizeof (VisSongInfo));

	songinfo->interface = interface;

	return songinfo;
}

int visual_songinfo_free (VisSongInfo *songinfo)
{
	if (songinfo == NULL)
		return -1;

	visual_songinfo_free_strings (songinfo);

	visual_video_free_with_buffer (songinfo->cover);

	free (songinfo);

	return 0;
}

int visual_songinfo_free_strings (VisSongInfo *songinfo)
{
	if (songinfo == NULL)
		return -1;

	if (songinfo->songname != NULL)
		free (songinfo->songname);

	if (songinfo->artist != NULL)
		free (songinfo->artist);

	if (songinfo->album != NULL)
		free (songinfo->album);

	if (songinfo->song != NULL)
		free (songinfo->song);

	return 0;
}

int visual_songinfo_set_interface (VisSongInfo *songinfo, int interface)
{
	if (songinfo == NULL)
		return -1;
	
	songinfo->interface = interface;

	return 0;
}

int visual_songinfo_set_length (VisSongInfo *songinfo, int length)
{
	if (songinfo == NULL)
		return -1;
	
	songinfo->length = length;

	return 0;
}

int visual_songinfo_set_elapsed (VisSongInfo *songinfo, int elapsed)
{
	if (songinfo == NULL)
		return -1;
	
	songinfo->elapsed = elapsed;

	return 0;
}

int visual_songinfo_set_simple_name (VisSongInfo *songinfo, char *name)
{
	if (songinfo == NULL)
		return -1;
	
	if (songinfo->songname != NULL)
		free (songinfo->songname);

	songinfo->songname = strdup (name);

	return 0;
}

int visual_songinfo_set_artist (VisSongInfo *songinfo, char *artist)
{
	if (songinfo == NULL)
		return -1;

	if (songinfo->artist != NULL)
		free (songinfo->artist);

	songinfo->artist = strdup (artist);

	return 0;
}

int visual_songinfo_set_album (VisSongInfo *songinfo, char *album)
{
	if (songinfo == NULL)
		return -1;

	if (songinfo->album != NULL)
		free (songinfo->album);

	songinfo->album = strdup (album);

	return 0;
}

int visual_songinfo_set_song (VisSongInfo *songinfo, char *song)
{
	if (songinfo == NULL)
		return -1;

	if (songinfo->song != NULL)
		free (songinfo->song);

	songinfo->song = strdup (song);

	return 0;
}

int visual_songinfo_set_cover (VisSongInfo *songinfo, VisVideo *cover)
{
	if (songinfo == NULL)
		return -1;

	songinfo->cover = cover;

	return 0;
}

int visual_songinfo_mark (VisSongInfo *songinfo)
{
	if (songinfo == NULL)
		return -1;
	
	songinfo->start = time (NULL);

	return 0;
}

time_t visual_songinfo_age (VisSongInfo *songinfo)
{
	time_t cur;

	cur = time (NULL);

	/* Clock has been changed */
	if (cur < songinfo->start)
		songinfo->start = cur;

	return cur - songinfo->start;
}

int visual_songinfo_copy (VisSongInfo *dest, VisSongInfo *src)
{
	dest->interface = src->interface;
	dest->length = src->length;
	dest->elapsed = src->elapsed;
	dest->start = src->start;

	if (src->songname != NULL)
		dest->songname = strdup (src->songname);
	
	if (src->artist != NULL)
		dest->artist = strdup (src->artist);
	
	if (src->album != NULL)
		dest->album = strdup (src->album);
	
	if (src->song != NULL)
		dest->song = strdup (src->song);

	/* FIXME copy the cover art video as well */

	return 0;
}

int visual_songinfo_compare (VisSongInfo *s1, VisSongInfo *s2)
{
	int diff = 0;

	if (s1->songname != NULL && s2->songname != NULL) {
		if (strcmp (s1->songname, s2->songname) != 0)
			diff++;
	}

	if (s1->artist != NULL && s2->artist != NULL) {
		if (strcmp (s1->artist, s2->artist) != 0)
			diff++;
	}

	if (s1->album != NULL && s2->album != NULL) {
		if (strcmp (s1->album, s2->album) != 0)
			diff++;
	}

	if (s1->song != NULL && s2->song != NULL) {
		if (strcmp (s1->song, s2->song) != 0)
			diff++;
	}
	
	return diff;
}

