#ifndef _LV_VIDEO_H
#define _LV_VIDEO_H

#include <libvisual/lv_palette.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */

/* NOTE: The depth find helper code in lv_actor depends on an arrangment from low to high */
#define VISUAL_VIDEO_CONTEXT_NONE	0
#define VISUAL_VIDEO_CONTEXT_8BIT	1
#define VISUAL_VIDEO_CONTEXT_16BIT	2
#define VISUAL_VIDEO_CONTEXT_24BIT	4
#define VISUAL_VIDEO_CONTEXT_32BIT	8
#define VISUAL_VIDEO_CONTEXT_GL		16
#define VISUAL_VIDEO_CONTEXT_ENDLIST	32

#define VISUAL_VIDEO_CONTEXT_ALL	VISUAL_VIDEO_CONTEXT_8BIT | VISUAL_VIDEO_CONTEXT_16BIT | VISUAL_VIDEO_CONTEXT_24BIT | VISUAL_VIDEO_CONTEXT_32BIT | VISUAL_VIDEO_CONTEXT_GL

typedef struct _VisVideo VisVideo;

struct _VisVideo { 
	int		 depth;
	int		 width;
	int		 height;
	int		 bpp;
	int		 size;
	int		 pitch;

	void		*screenbuffer;
	VisPalette	*pal;
};

/* prototypes */
VisVideo *visual_video_new ();
VisVideo *visual_video_new_with_buffer (int width, int height, int depth);
int visual_video_free (VisVideo *video);
int visual_video_free_with_buffer (VisVideo *video);
int visual_video_free_buffer (VisVideo *video);
int visual_video_allocate_buffer (VisVideo *video);
int visual_video_clone (VisVideo *dest, VisVideo *src);

int visual_video_set_palette (VisVideo *video, VisPalette *pal);
int visual_video_set_buffer (VisVideo *video, void *buffer);
int visual_video_set_dimension (VisVideo *video, int width, int height);
int visual_video_set_pitch (VisVideo *video, int pitch);
int visual_video_set_depth (VisVideo *video, int depth);

int visual_video_depth_is_supported (int depthflag, int depth);
int visual_video_depth_get_next (int depthflag, int depth);
int visual_video_depth_get_previous (int depthflag, int depth);
int visual_video_depth_get_lowest (int depthflag);
int visual_video_depth_get_highest (int depthflag);
int visual_video_depth_get_highest_nogl (int depthflag);
int visual_video_depth_is_sane (int depth);
int visual_video_depth_value_from_enum (int enumval);
int visual_video_depth_enum_from_value (int depth);

int visual_video_bpp_from_depth (int enumval);

int visual_video_fit_in_video (VisVideo *dest, VisVideo *src);

int visual_video_depth_transform (VisVideo *viddest, VisVideo *vidsrc);
int visual_video_depth_transform_to_buffer (unsigned char *dest, VisVideo *video,
		VisPalette *pal, int destdepth, int pitch);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* _LV_VIDEO_H */
