#include <unistd.h>
#include <stdlib.h>

#include <time.h>
#include <sys/time.h>

#include <SDL/SDL.h>

#include <libvisual.h>

SDL_Surface *screen;
SDL_Color colors[256];

void sdl_init (int bpp)
{
	if (SDL_Init (SDL_INIT_VIDEO) < 0)
	{
		fprintf (stderr, "Unable to init SDL VIDEO: %s\n", SDL_GetError ());
		exit (0);
	}

	screen = SDL_SetVideoMode (320, 200, bpp, SDL_RESIZABLE);
}

void sdl_draw (VisVideo *video, unsigned char *buf, int unit1)
{
	unsigned char *str = (unsigned char *) screen->pixels;
	
	memcpy (str, buf, video->width * video->height * unit1);

	SDL_UpdateRect (screen, 0, 0, 0, 0);
}

void sdl_set_pal (VisPalette *pal)
{
	int i;

	for (i = 0; i < 256; i ++) {
		colors[i].r = pal->r[i];
		colors[i].g = pal->g[i];
		colors[i].b = pal->b[i];
	}
	
	SDL_SetColors (screen, colors, 0, 256);
}

int main (int argc, char *argv[])
{
	VisPluginRef *ref;
	
	VisActorPlugin *actor;
	VisPluginInfo *info;
	VisVideo *video, *tmpvid;
	VisPalette *pal;
	unsigned char *tmpbuf;
	unsigned char *scrbuf;
	unsigned char *trnbuf;
	unsigned char *temp;

        time_t begin = time (NULL), end;
		

	VisInput *input;
	VisList *inlist;
	
	VisList *list;
	int i, j = 8;
	int unit1 = 1;
	int realdepth = VISUAL_VIDEO_CONTEXT_8BIT;
	int surfdepth1 = VISUAL_VIDEO_CONTEXT_8BIT;

	int dtrans = 0;
	int unit2;
	int surfdepth2;

	int transform = 0;
	int freeze = 0;

	int frames = 0;

	SDL_Event event;

	inlist = visual_input_get_list (NULL);
	input = visual_input_new (inlist, "esd");
	visual_input_realize (input);

	list = _lv_plugin_actor_get_list ("/usr/lib/libvisual/actor");
	
	if (argc > 1)
		ref = _lv_plugin_actor_find (list, argv[1]);
	else
		ref = _lv_plugin_actor_find (list, "infinite");

	actor = _lv_plugin_actor_load (ref);
	_lv_plugin_actor_realize (actor);

	printf ("Actor %p\n", actor);
	if (actor == NULL)
		exit (-1);

	for (i = 1; i < 16; ) {
		printf ("Bits %d: supported %d, %d\n", j, (i & actor->depth) > 0 ? 1 : 0, actor->depth);
		j += 8;
		i *= 2;
	}

	if (actor->depth & VISUAL_VIDEO_CONTEXT_GL)
		printf ("Ooh and we also support OPENGL!\n");

	if (argc > 2) {
		i = atoi (argv[2]);
		printf ("selected %d depth\n", i);

		surfdepth1 = visual_video_depth_enum_from_value (i);

		if (surfdepth1 == -1) {
			printf ("Stupid depth value, go figure! :)\n");
			exit (-1);
		}

		unit1 = visual_video_bpp_from_enum (surfdepth1);
	}

	if (argc > 3) {
		i = atoi (argv[3]);
		printf ("dest depth %d\n", i);

		surfdepth2 = visual_video_depth_enum_from_value (i);

		if (surfdepth2 == -1) {
			printf ("Stupid dest depth\n");
			exit (-1);
		}
		
		unit2 = visual_video_bpp_from_enum (surfdepth2);
		
		dtrans = 1;
	}
	
	scrbuf = malloc (320 * 200 * unit1);
	memset (scrbuf, 0, 320 * 200 * unit1);

	if ((surfdepth1 & actor->depth) == 0) {
		printf ("Plugin doesn't support this depth, enabling depth transformation\n");
		transform = 1;
		realdepth = VISUAL_VIDEO_CONTEXT_8BIT;
		tmpbuf = malloc (320 * 200 * unit1);
		memset (tmpbuf, 0, 320 * 200 * unit1);
	} else {
		realdepth = surfdepth1;
		tmpbuf = scrbuf;
	}

	if (dtrans == 1)
		trnbuf = malloc (320 * 200 * unit2);

	video = visual_video_new ();
	visual_video_set_opts (video, "depth", realdepth);
	visual_video_set_dimension (video, 320, 200);
	visual_video_set_buffer (video, tmpbuf);

	tmpvid = visual_video_new ();
	visual_video_set_opts (tmpvid, "depth", surfdepth1);
	visual_video_set_dimension (tmpvid, 320, 200);

	if (dtrans == 1)
		sdl_init (unit2 * 8);
	else
		sdl_init (unit1 * 8);

	actor->dimension (actor, video, 320, 200);

	if (video->depth != VISUAL_VIDEO_CONTEXT_8BIT)
		pal = visual_palette_new ();
	
	while (1) {
		visual_input_run (input);

		actor->render (actor, video, input->audio);

		if (video->depth == VISUAL_VIDEO_CONTEXT_8BIT)
			pal = actor->palette (actor);
		
		if (pal != NULL)
			sdl_set_pal (pal);
	
		if (transform == 1)
			visual_video_depth_transform_to_buffer (scrbuf, video, pal, surfdepth1);
		
		if (freeze == 0) {
			if (dtrans == 1) {
				visual_video_set_dimension (tmpvid, video->width, video->height);
				visual_video_set_buffer (tmpvid, scrbuf);
			
				visual_video_depth_transform_to_buffer (trnbuf, tmpvid, pal, surfdepth2);
				
				sdl_draw (video, trnbuf, unit2);
			} else {
				sdl_draw (video, scrbuf, unit1);
			}
			frames++;
		}
		
		
		usleep (5000);

		while (SDL_PollEvent (&event))
		{
			switch (event.type)
			{
				case SDL_KEYDOWN:
					switch (event.key.keysym.sym)
					{
						case SDLK_f:
							freeze = 1 - freeze;
							break;
							
						case SDLK_F11:
							SDL_WM_ToggleFullScreen (screen);
							break;

						case SDLK_p:
							end = time (NULL);

							printf ("Drawn %d frames in %d seconds, average fps %d\n",
									frames, end - begin, frames / (end - begin));
							break;
							
						case SDLK_ESCAPE:
							goto out;
							break;
					}
					break;

				case SDL_VIDEORESIZE:
					SDL_FreeSurface (screen);

					actor->dimension (actor, video, event.resize.w, event.resize.h);

					free (scrbuf);
					scrbuf = malloc (video->width * video->height * unit1);
					memset (scrbuf, 0, video->width * video->height * unit1);

					if (dtrans) {
						free (trnbuf);
						trnbuf = malloc (video->width * video->height * unit2);
					}
					
					if (transform == 1) {
						free (tmpbuf);

						tmpbuf = malloc (video->size);
						memset (tmpbuf, 0, video->size);

					} else {
						tmpbuf = scrbuf;
					}

					video->screenbuffer = tmpbuf;
					
					if (dtrans == 1)
						screen = SDL_SetVideoMode (video->width, video->height, unit2 * 8, SDL_RESIZABLE);
					else
						screen = SDL_SetVideoMode (video->width, video->height, unit1 * 8, SDL_RESIZABLE);

					break;

				case SDL_QUIT:
					goto out;
					break;
			}
		}
	}
out:
	end = time (NULL);

	printf ("Drawn %d frames in %d seconds, average fps %d\n",
			frames, end - begin, frames / (end - begin));

	SDL_Quit ();
	
	visual_input_destroy (input);
	_lv_plugin_actor_unload (actor);

	visual_input_list_destroy (inlist);
}

