#include <stdio.h>
#include <signal.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <linux/fb.h>
#include <sys/ioctl.h>
#include <sys/mman.h>

#include <pthread.h>

#include <libvisual.h>

static struct fb_fix_screeninfo fbfix;
static struct fb_var_screeninfo fbvar;

static unsigned char *fbmem;
static int fbdev;

short int snddata[1024];
short int olddata[1024];

VisAudio audio;

int screen_xsize = 420;
int screen_ysize = 200;

char *eyecandy = "jess";
int depth = 8;

static void inline fb_put_pixel (int x, int y, unsigned char r, unsigned char g, unsigned char b)
{
	fbmem[(((y * fbvar.xres) + x) * 4)] = r;
	fbmem[(((y * fbvar.xres) + x) * 4) + 1] = g;
	fbmem[(((y * fbvar.xres) + x) * 4) + 2] = b;
}

void fb_put_32_buf (unsigned char *buf, int width, int height)
{
	int i, j;
	unsigned char r, g, b, ind = 0;

	for (i = 0; i < height; i++) {
		for (j = 0; j < width; j++) {
			r = buf[ind++];
			g = buf[ind++];
			b = buf[ind++];
			
			fb_put_pixel ((fbvar.xres - width) + j, i, r, g, b);
		}
	}
}

void fb_put_pal_buf (unsigned char *buf, VisPalette *pal, int width, int height)
{
	int i, j;
	unsigned char r, g, b, ind;

	for (i = 0; i < height; i++) {
		for (j = 0; j < width; j++) {
			ind = buf[(i * width) + j];

			r = pal->r[ind];
			g = pal->g[ind];
			b = pal->b[ind];
		
			fb_put_pixel ((fbvar.xres - width) + j, i, r, g, b);
		}
	}
}

void *render_thread (void *arg)
{
	VisPalette *pal;
	VisPluginRef *ref;
	VisActorPlugin *actor;
	VisVideo *video;
	VisList *list;
	unsigned char *buf;

	buf = malloc (screen_xsize * screen_ysize);
	memset (buf, 0, screen_xsize * screen_ysize);

	video = visual_video_new ();
	visual_video_set_opts (video, "depth",
			visual_video_depth_enum_from_value (depth));
	
	visual_video_set_dimension (video, screen_xsize, screen_ysize);
	visual_video_set_buffer (video, buf);

	list = _lv_plugin_actor_get_list ("/usr/lib/libvisual/actor");

	ref = _lv_plugin_actor_find (list, eyecandy);

	actor = _lv_plugin_actor_load (ref);
	_lv_plugin_actor_realize (actor);
	actor->dimension (actor, video, screen_xsize, screen_ysize);

	free (buf);
	buf = malloc (video->width * video->height);
	memset (buf, 0, video->width * video->height);
	
	while (1) {
		visual_audio_analyze (&audio);

		actor->render (actor, video, &audio);

		pal = actor->palette (actor);

		if (depth == 8)
			fb_put_pal_buf (buf, pal, video->width, video->height);
		else if (depth == 32)
			fb_put_32_buf (buf, video->width, video->height);
	}
}

void *audio_thread (void *arg)
{
	int i;
	int readn = sizeof (snddata);

	memset (snddata, 0, sizeof (snddata));

	while (1) {
		read (0, snddata, readn);
		write (1, snddata, readn);

		for (i = 0; i < 1024; i += 2) {
			audio.plugpcm[0][i >> 1] = olddata[i];
			audio.plugpcm[1][i >> 1] = olddata[i + 1];
		}

		read (0, snddata, readn);
		write (1, snddata, readn);

		memcpy (olddata, snddata, sizeof (olddata));
	}
}

int main (int argc, char *argv[])
{
	pthread_t render_thr;
	pthread_t audio_thr;
	
	fbdev = open("/dev/fb0", O_RDWR);

	ioctl(fbdev, FBIOGET_FSCREENINFO, &fbfix);
	ioctl(fbdev, FBIOGET_VSCREENINFO, &fbvar);
	
	fbmem = mmap(NULL,(unsigned long)fbfix.smem_len,PROT_WRITE,MAP_SHARED,fbdev,0);

	if (argc > 1)
		eyecandy = argv[1];

	if (argc > 2)
		depth = atoi (argv[2]);

	pthread_create (&render_thr, NULL, render_thread, NULL);
	pthread_create (&audio_thr, NULL, audio_thread, NULL);

	/* Your mother likes this as well, seriously */
	while (1)
		sleep (100000);
}

