#include <stdio.h>
#include <stdlib.h>

#include <time.h>
#include <sys/time.h>

#include <GL/gl.h>
#include <GL/glu.h>
#include <SDL/SDL.h>

#include <libvisual.h>

#define SCREEN_WIDTH	320
#define SCREEN_HEIGHT	200
#define SCREEN_BPP	16

#define TRUE	1
#define FALSE	0

SDL_Surface *screen;

void window_quit (int retcode)
{
	SDL_Quit ();
	exit (retcode);
}

int main (int argc, char **argv)
{	
	VisPluginRef *ref;
	VisActorPlugin *actor;
	VisVideo *video;
	VisList *list;

	VisInput *input;
	VisList *inlist;

	int videoflags;
	int done = FALSE;
	SDL_Event event;
	const SDL_VideoInfo *videoinfo;
	int active = TRUE;
	char blend = FALSE;
	float scene_z = -6;
	char key_up = FALSE;
	char key_down = FALSE;
	
	int begin = time (NULL), end;
	int frames = 0;

	int pauze = 0;
	
	if (SDL_Init (SDL_INIT_VIDEO) < 0)
		window_quit (1);
		
	videoinfo = SDL_GetVideoInfo ();
	
	if (videoinfo == 0)
		window_quit (1);
	
	videoflags = SDL_OPENGL | SDL_GL_DOUBLEBUFFER | SDL_HWPALETTE | SDL_RESIZABLE;
	
	if (videoinfo->hw_available)
	{
		printf ("VIDEO: Hardware surface\n");
		videoflags |= SDL_HWSURFACE;
	}
	else
	{
		printf ("VIDEO: Software surface\n");
		videoflags |= SDL_SWSURFACE;
	}
		
	if (videoinfo->blit_hw)
	{
		printf ("VIDEO: Hardware accel\n");
		videoflags |= SDL_HWACCEL;
	}

	printf ("DEBUG: Setting SDL_GL_DOUBLEBUFFER attribute\n");	
	SDL_GL_SetAttribute (SDL_GL_DOUBLEBUFFER, 1);
	
	printf ("DEBUG: Videomode setting up\n");
	screen = SDL_SetVideoMode (SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP, videoflags);
	
	printf ("DEBUG: Checking if screen isn't NULL\n");
	if (screen == 0)
		window_quit (1);

	printf ("DEBUG: Initial window resizement\n");
	
	printf ("DEBUG: Entering while loop\n");

	/* Libvisual init stuff */
	printf ("Opening lists\n");
	inlist = visual_input_get_list (NULL);
	input = visual_input_new (inlist, "esd");
	visual_input_realize (input);

	list = _lv_plugin_actor_get_list ("/usr/lib/libvisual/actor");


	printf ("Loading actor\n");
	if (argc > 1)
		ref = _lv_plugin_actor_find (list, argv[1]);
	else
		ref = _lv_plugin_actor_find (list, "lv_gltest");

	actor = _lv_plugin_actor_load (ref);
	_lv_plugin_actor_realize (actor);

	if (actor == NULL) {
		printf ("Dudeeeee, actor is NULL!\n");
		exit (-1);
	}
	if (!(actor->depth & VISUAL_VIDEO_CONTEXT_GL)) {
		printf ("Not an GL plugin\n");
		exit (-1);
	}

	video = visual_video_new ();
	visual_video_set_opts (video, "depth", VISUAL_VIDEO_CONTEXT_GL);
	visual_video_set_dimension (video, 320, 200);
	visual_video_set_buffer (video, NULL);

	printf ("Have set video opts now loading plugins\n");

	/* Start renderer and loop and such */
	printf ("Setting initial dimension\n");
	actor->dimension (actor, video, 320, 200);
	
	printf ("Starting the render loop\n");
	while (!done)
	{
		visual_input_run (input);
		
		if (pauze == 0) {
			actor->render (actor, video, input->audio);

			SDL_GL_SwapBuffers ();
			frames++;
		}
		
		usleep (10000);

		while (SDL_PollEvent (&event))
		{
			switch (event.type)
			{
			case SDL_VIDEORESIZE:
				actor->dimension (actor, video, event.resize.w, event.resize.h);

				screen = SDL_SetVideoMode (video->width, video->height, SCREEN_BPP, videoflags);
				
				if (screen == 0)
					window_quit (1);
			
				break;
				
			case SDL_KEYDOWN:
				switch (event.key.keysym.sym)
				{
				case SDLK_ESCAPE:
					done = TRUE;
					break;
					
				case SDLK_F11:
					SDL_WM_ToggleFullScreen (screen);
					break;
				
				case SDLK_f:
					pauze = 1 - pauze;
					break;
				
				case SDLK_b:
					blend = 1 - blend;
					
					if (blend == TRUE) {
						glEnable (GL_BLEND);
						glDisable (GL_DEPTH_TEST);
					} else {
						glDisable (GL_BLEND);
						glEnable (GL_DEPTH_TEST);
					}
					
					break;
			
				case SDLK_UP:
					key_up = TRUE;
					break;

				case SDLK_DOWN:
					key_down = TRUE;
					break;
					
				case SDLK_p:
					end = time (NULL);

					printf ("Drawn %d frames in %d seconds, average fps %d\n",
							frames, end - begin, frames / (end - begin));
					break;

				default:
					break;
				}
				break;
								
			case SDL_KEYUP:
				switch (event.key.keysym.sym)
				{
				case SDLK_UP:
					key_up = FALSE;
					break;
				
				case SDLK_DOWN:
					key_down = FALSE;
					break;
				
				default:
					break;
				}
				break;

			case SDL_QUIT:
				done = TRUE;
				break;
				
			default:
				break;
			}
		}
		
		if (key_up == TRUE)
			scene_z += 0.2;
		
		if (key_down == TRUE)
			scene_z -= 0.2;
	}

	end = time (NULL);

	printf ("Drawn %d frames in %d seconds, average fps %d\n",
			frames, end - begin, frames / (end - begin));

	visual_video_free (video);

	SDL_Quit ();

	visual_input_destroy (input);
	_lv_plugin_actor_unload (actor);

	visual_input_list_destroy (inlist);

	window_quit (0);

	return 0;
}

